#ifndef TGRAPHICS_OBJECT_H
#define TGRAPHICS_OBJECT_H

#include "GraphicsAPI.h"
#include "TxformInfo.h"
#include "Tmaterial.h"

		//
		// Set and get attr/function for compound data type:
		//  	e.g. D3DXCOLOR
		//
#define CompoundObjAttr(_dataType_, _attrName_)									\
			protected:															\
					_dataType_	f##_attrName_##;								\
			public:																\
					virtual void Set##_attrName_##(const _dataType_ &na); 		\
					virtual _dataType_##& Get##_attrName_##();  				\

#define ImplementCompoundObjAttr(_className_, _dataType_, _attrName_)			\
				void _className_##::Set##_attrName_##(const _dataType_ &na) {	\
						f##_attrName_## = na; }									\
				_dataType_##& ##_className_##::Get##_attrName_##() { 			\
						return f##_attrName_##; }								\

			
		//
		// Set and get attr/function for simple data type:
		//  	e.g. D3DXCOLOR
		//
#define SimpleObjAttr(_dataType_, _attrName_)									\
			protected:															\
					_dataType_	f##_attrName_##;								\
			public:																\
					virtual void Set##_attrName_##(const _dataType_ na); 		\
					virtual _dataType_ Get##_attrName_##() const;   			\
			
#define ImplementSimpleObjAttr(_className_, _dataType_, _attrName_)				\
				void _className_##::Set##_attrName_##(const _dataType_ na) {	\
							f##_attrName_## = na; }								\
				_dataType_  _className_##::Get##_attrName_##() const 	{ 		\
							return f##_attrName_##; }							\
			

		//
		// Set and get attr/function for boolean data type:
		//  	e.g. D3DXCOLOR
		//
#define BoolObjAttr(_attrName_)													\
			protected:															\
					bool	f##_attrName_##;									\
			public:																\
					virtual void Enable##_attrName_##(); 						\
					virtual void Disable##_attrName_##(); 						\
					virtual bool  _attrName_##IsEnabled() const;	  			\

#define ImplementBoolObjAttr(_className_, _attrName_)							\
				void _className_##::Enable##_attrName_##() {					\
							f##_attrName_## = true; }							\
				void _className_##::Disable##_attrName_##() {					\
							f##_attrName_## = false; }							\
				bool  _className_##::_attrName_##IsEnabled() const 	{ 			\
							return f##_attrName_##; }							\
	
			

	// the origin for all graphics objects are located at
	// lower left-hand corner. Caller must ensure this is obeyed.
	//
class TgraphicsObject {

	private:
		void InitializeGraphicsObject();

	protected:
		//
		// TxformInfo fXformInfo; // Transformation informaiton
		CompoundObjAttr(TxformInfo, XformInfo)

		//
		// D3DXVECTOR3	fVelocity; // speed
		CompoundObjAttr(D3DXVECTOR3, Velocity)

		// Tmaterial	fMaterial;	// material of the object
		//							// when fLighting is off, fMaterial.fm.Emission is the drawing color
		CompoundObjAttr(Tmaterial, Material)
		
		//
		// bool fHiLight; // Is this object highlighted?
		//
		BoolObjAttr(HiLight)

		//
		// bool fVisibility; // is this object visible?
		BoolObjAttr(Visibility)

		
		// 
		// bool fShowAxisFrame;  // should we display the reference frame of this object?
		BoolObjAttr(ShowAxisFrame)

		//
		// bool fLighting;		// should we using the hw lighting equation?
		BoolObjAttr(Lighting)

		//
		//
		// Following methods are for subclass to override
		// which will define/modify the detailed behavior
		// of each types of objects
		//
		//
		//
	
	
		virtual bool ShouldDrawObject() const;
					// should this object be drawn (e.g. is it visible?)

		virtual void SetObjectD3DAttributes(Td3dDrawable &);
					// subclass is responsible for setting the object's attributes
		
		virtual void UnsetObjectD3DAttributes( Td3dDrawable&);
					// subclass is responsible for un-setting the object's attributes
					
					
		virtual void DrawObjectGeom(Td3dDrawable&, LPDIRECT3DDEVICE9);
					// draws the object geometry
					// subclasses are responsible for
					// defining the implementaion of
					// this method.

		virtual bool RayPickObjectGeom(Td3dDrawable&, LPDIRECT3DDEVICE9, const D3DXVECTOR3&, const D3DXVECTOR3&);
					// Returns if input rays picks the geometry
					// subclasses are responsible for
					// defining the implementaion of
					// this method.

		virtual void EchoHighLight(LPDIRECT3DDEVICE9) const;
					// if high-lighted is true, how should object
					// echo this?
					//
				
	public:
		virtual ~TgraphicsObject();
		TgraphicsObject();

		virtual void SetLocationTo(float x, float y, float z);
		void SetScale(float x, float y, float z);
		void SetScale(float s);
		void SetRotation(float r);
		void SetRotation(float x, float y, float z);

		virtual const D3DXVECTOR3& GetLocation() const;
		const D3DXVECTOR3& GetScale() const;
		const D3DXVECTOR3& GetRotation() const;

				
		void DrawObject(Td3dDrawable &drawable);
		bool RayPickObject(Td3dDrawable &drawable, const D3DXVECTOR3&, const D3DXVECTOR3&);

		
		//
		virtual void SetObjectD3DTransformation(ID3DXMatrixStack *stack) const;

		virtual void UpdateLocation();
					//
					//  Updates location by fVelocity
					//  

		//
		//
		// Following are virtual functions that describe behaviors
		// of drawable objects.  appropriate subclasses
		// should override these functions.
		//
		// the following set of behaviors allows TgraphicsObject understand
		// the language of all subclasses.  This is important for
		// "generic" graphics operation, e.g. selection in TobjectList
		//
		// 
				//  point size
		virtual void SetPointSize(const float s);

				// fillable attribute
		virtual void SetFillMode(const WORD);
		virtual WORD GetFillMode() const { return D3DFILL_SOLID; }

				// line and fillable attribute
		virtual void SetShadedMode(const WORD);
		virtual WORD GetShadedMode() const { return D3DSHADE_GOURAUD; }
				//
				// attributes
		virtual void SetDrawColor(const D3DXCOLOR&);

				// texture index
		virtual void SetTextureIndex(const int i);

				// Alpha Blending
		virtual void EnableAlphaBlending() {}
		virtual void DisableAlphaBlending() {}
		virtual bool AlphaBlendingIsEnabled() const { return false; }

				// Show Normal
		virtual void EnableShowNormal() {}
		virtual void DisableShowNormal() {}
		virtual bool ShowNormalIsEnabled() const { return false; }

				// Show Control Grid (for mesh)
		virtual void EnableShowControlGrid() {}
		virtual void DisableShowControlGrid() {}
		virtual bool ShowControlGridIsEnabled() const { return false; }

		
				// Show Texture
		virtual void EnableShowTexture() {}
		virtual void DisableShowTexture() {}
		virtual bool ShowTextureIsEnabled() const { return false; }
};

#endif
